#!/usr/bin/env python
# -*- coding: utf-8 -*-

from gimpfu import *
import subprocess
import sys, os, gettext
from subprocess import call

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#   The GPL v3 licence is available at: https://www.gnu.org/licenses/gpl-3.0.en.html

# ------------
#| Change Log |
# ------------
# Rel 0.1: Initial release.
# This plugin based on ofn-export-layers.py by Ofnuts: https://sourceforge.net/projects/gimp-tools/files/scripts/
# and information contained on: ImageMagick Examples - Animation Basics http://www.imagemagick.org/Usage/anim_basics/
# Download:ImageMagick-7.0.9-14-Q16-x64-dll.exe https://imagemagick.org/script/download.php#windows
# Rel 0.2 - "\" has been replaced on os.sep,
#         - extended "Alpha Channel (for layers) function: Remove, Add, Semiflaten.

whoiam = os.path.abspath(sys.argv[0])
locale_directory = os.path.join(os.path.dirname(whoiam), 'locale')
gettext.install( "Animated_gif_using_ImageMagick", locale_directory, unicode=True )

# ============== Start Export Layer by code (c) Ofnuts ===============================================================================
def exportLayers_gif(image,directory,namePattern):

    try:
        if not image.filename:
            imageName='Layer'
        else:
            imageName,_=os.path.splitext(os.path.basename(image.filename))
        layersCount=len(image.layers)
        formatValues={}
        formatValues['imageName']=imageName
        formatValues['count']=layersCount
        for i,layer in enumerate(image.layers):   # rename image.layers to apply_layers
            formatValues['numDown0']=i
            formatValues['numDown1']=i+1
            formatValues['numUp0']=layersCount-(i+1)
            formatValues['numUp1']=layersCount-i
            formatValues['rawName']=layer.name
            formatValues['name']=os.path.splitext(layer.name)[0]
            formatValues['width']=layer.width
            formatValues['height']=layer.height
    
            try:
                filename=namePattern.format(**formatValues)
            except KeyError as e:
                raise Exception('No formatting variable called "%s"' % e.args[0])        
            filename=os.path.join(directory,filename)
            pdb.gimp_file_save(image,layer,filename,filename)
            
    except Exception as e:
        pdb.gimp_message(e.args[0])
        print traceback.format_exc()
# ============== End Export Layer by code (c) Ofnuts ===============================================================================

def sys_file(f_name):
    sys_encoded = sys.getfilesystemencoding()
    if os.name == 'nt': coded = f_name.encode(sys_encoded)
    else: coded = f_name
    return(coded)
	
def plugin_main(image, layer, convert, remalpha, order, directory, delayParam,disposeParam, duplicateParam, gifname, rem):

	convert_dirpath = sys_file(os.path.join(convert))
	output_dirpath = sys_file(os.path.join(directory))
	
	pdb.gimp_image_undo_group_start(image)
	
	if remalpha==1:
		for layer in image.layers:
			pdb.gimp_image_set_active_layer(image,layer)
			pdb.gimp_layer_flatten(layer)
	if remalpha==2:
		for layer in image.layers:
			pdb.gimp_image_set_active_layer(image,layer)
			pdb.gimp_layer_add_alpha(layer)
	if remalpha==3:
		for layer in image.layers:
			pdb.plug_in_semiflatten(image,layer)
			
	pdb.gimp_image_undo_group_end(image)
	
	if order == 0:
		namePattern='{numDown1:03d}.png'
	if order == 1:
		namePattern='{numUp1:03d}.png'
		
	exportLayers_gif(image,directory,namePattern)
	
	path1 = sys_file(output_dirpath + os.sep + '*.png')
	path2 = sys_file(output_dirpath + os.sep + gifname + '.gif')

	if disposeParam == 0:
		dis = 'Background'
	if disposeParam == 1:
		dis = 'None'  

	if duplicateParam == 0:
		cmd = [convert_dirpath, '-delay', str(delayParam), '-dispose', str(dis),'-loop', '0', path1, path2]	
	if duplicateParam == 1:
		pingpong = '1,-2-1'
		cmd = [convert_dirpath, '-delay', str(delayParam), '-dispose', str(dis),'-duplicate', str(pingpong),'-loop', '0', path1, path2]

	print cmd
	call(cmd)	
	
	if rem == 1:
		filelist = [ f for f in os.listdir(output_dirpath) if f.endswith('.png') ] 
		for f in filelist: 
			os.remove(os.path.join(output_dirpath, f)) 

register(
		"python_fu_amimated_gif",
		"Create gif using ImageMagick (must be installed).\nNames cannot contain accented characters or spaces.",
		"Create gif animation using ImageMagick",
		"MrQ",
		"GPL",
		"2020",
		#"<Image>/ImageMagick/Animated gif using ImageMagick...",		
		"<Image>/Filters/Animation/Animated gif using ImageMagick...", # menuitem with accelerator key
		"*", # image types
		[
		(PF_FILENAME, "convert", " ImageMagic convert.exe file: ", "c:\\Program Files\\ImageMagick-7.0.9-Q16\\convert.exe"),
#Linux  (PF_FILENAME, "convert", " ImageMagic convert file: ",'usr/bin/convert'),
		(PF_OPTION, "remalpha",  " Change alpha chanels\n (for all layers): ",0,[" DON'T USE ", " REMOVE ", " ADD ", " SEMIFLATEN "]),			
		(PF_OPTION, "order",  " Play order: ",1,[" From top to bottom "," From bottom to top "]),
		(PF_DIRNAME, "directory",    " Directory to save png and gif:\n (create or select an EMPTY)",   "."),
		(PF_SPINNER, "delayParam",    " Delay [100 = 1s]:", 100, (1, 10000, 1)),
		(PF_OPTION, "disposeParam",  " Frame Disposal Methods: ",1,[" Background (replace) ", " None (combine) "]),
		(PF_TOGGLE, "duplicateParam",   " Ping-Pong ?: ", 0),
		(PF_STRING, "gifname",  " Gif file name: ", "GimpChat"),
		(PF_TOGGLE, "rem",   " Remove png files?\n (all in Directory): ", 0),		
		],
		[],
		plugin_main,
		)

main()
 
